@@ -8,10 +8,10 @@
 #define LIFETIME_TRESHOLD 10*24*60*60 //10 days
 
 static STACK_OF(X509)* my_sk_X509_dup(glb_ctx *cc, STACK_OF(X509) *stack);
-extern int proxy_verify_cert_chain(X509 * ucert, STACK_OF(X509) * cert_chain, proxy_verify_desc * pvd);
-extern proxy_verify_desc *pvd_setup_initializers(char *cadir, 
-        unsigned int flags);
-extern void pvd_destroy_initializers(void *data);
+extern int proxy_verify_cert_chain(X509 * ucert, STACK_OF(X509) * cert_chain, canl_proxy_verify_desc * pvd);
+extern canl_proxy_verify_desc *canl_pvd_setup_initializers(char *cadir, 
+        char *ocsp_url, unsigned int flags);
+extern void canl_pvd_destroy_initializers(void *data);
 extern canl_error map_verify_result(unsigned long ssl_err,
                 const X509_STORE_CTX *store_ctx, SSL *ssl);
 
@@ -375,13 +375,16 @@ canl_cred_set_cert_type(canl_ctx ctx, canl_cred cred,
 
 /*TODO use flags*/
 canl_err_code CANL_CALLCONV
-canl_cred_sign_proxy(canl_ctx ctx, canl_cred signer_cred, canl_cred proxy_cred)
+canl_cred_sign_proxy(canl_ctx ctx, canl_cred signer_cred, 
+                        canl_cred proxy_cred)
 {
     glb_ctx *cc = (glb_ctx*) ctx;
     creds *signer_crd = (creds*) signer_cred;
     creds *proxy_crd = (creds*) proxy_cred;
     int err = 0;
     int key_size = 0;
+    enum canl_cert_type cert_type = 0;
+    int proxyver;
 
     if (!ctx)
         return EINVAL;
@@ -407,10 +410,21 @@ canl_cred_sign_proxy(canl_ctx ctx, canl_cred signer_cred, canl_cred proxy_cred)
                    "respect to cert. lifetime");
     }
 
+    switch (proxy_crd->c_type){
+        case CANL_EEC:
+             proxyver = 2;
+             break;
+        case CANL_RFC:
+             proxyver = 3;
+             break;
+        default:
+             proxyver = 2;
+             break;
+    }
     /*TODO flags - limited,version*/
     err = proxy_sign(signer_crd->c_cert, signer_crd->c_key, proxy_crd->c_req,
             &proxy_crd->c_cert, proxy_crd->c_lifetime, 
-            proxy_crd->c_cert_ext, 0, 2, NULL, NULL, 0, NULL, 0);
+            proxy_crd->c_cert_ext, 0, proxyver, NULL, NULL, 0, NULL, 0);
     if (err)
         return set_error(cc, CANL_ERR_unknown, CANL_ERROR, "");
         
@@ -697,11 +711,11 @@ canl_verify_chain(canl_ctx ctx, X509 *ucert, STACK_OF(X509) *cert_chain,
         char *cadir)
 {
     int ret = 0;
-    proxy_verify_desc *pvd = NULL; /* verification context */
+    canl_proxy_verify_desc *pvd = NULL; /* verification context */
 
-    pvd = pvd_setup_initializers(cadir, 0);
+    pvd = canl_pvd_setup_initializers(cadir, NULL, 0);
     ret = proxy_verify_cert_chain(ucert, cert_chain, pvd);
-    pvd_destroy_initializers(pvd);
+    canl_pvd_destroy_initializers(pvd);
     if (ret)
         /* This will be ommited when proxy_verify_cert sets errors itself or
            propagate them out. */
@@ -716,16 +730,16 @@ canl_verify_chain_wo_ossl(canl_ctx ctx, char *cadir,
 {
     int ret = 0, depth = 0, i = 0;
     STACK_OF(X509) *certstack;
-    proxy_verify_desc *pvd = NULL; /* verification context */
+    canl_proxy_verify_desc *pvd = NULL; /* verification context */
     unsigned long ssl_err = 0;
     canl_error canl_err = 0;
 
-    pvd = pvd_setup_initializers(cadir, 0);
+    pvd = canl_pvd_setup_initializers(cadir, NULL, 0);
     X509_STORE_CTX_set_ex_data(store_ctx, PVD_STORE_EX_DATA_IDX, (void *)pvd);
 #ifdef X509_V_FLAG_ALLOW_PROXY_CERTS
     X509_STORE_CTX_set_flags(store_ctx, X509_V_FLAG_ALLOW_PROXY_CERTS);
 #endif
-    pvd_destroy_initializers(pvd);
+    canl_pvd_destroy_initializers(pvd);
 
     certstack = X509_STORE_CTX_get_chain(store_ctx);
     depth = sk_X509_num(certstack);
@@ -751,15 +765,16 @@ canl_verify_chain_wo_ossl(canl_ctx ctx, char *cadir,
     return 0;
 }
 
-proxy_verify_desc *pvd_setup_initializers(char *cadir, unsigned int pvxd_flags)
+canl_proxy_verify_desc *canl_pvd_setup_initializers(char *cadir, char *ocsp_url,
+        unsigned int pvxd_flags)
 {
-    proxy_verify_ctx_desc *pvxd = NULL;
-    proxy_verify_desc *pvd = NULL;
+    canl_proxy_verify_ctx_desc *pvxd = NULL;
+    canl_proxy_verify_desc *pvd = NULL;
     char *ca_cert_dirn = NULL;
     int err = 0;
 
-    pvd  = (proxy_verify_desc*)     malloc(sizeof(proxy_verify_desc));
-    pvxd = (proxy_verify_ctx_desc *)malloc(sizeof(proxy_verify_ctx_desc));
+    pvd  = (canl_proxy_verify_desc*)     malloc(sizeof(canl_proxy_verify_desc));
+    pvxd = (canl_proxy_verify_ctx_desc *)malloc(sizeof(canl_proxy_verify_ctx_desc));
     pvd->cert_store = NULL;
 
 
@@ -769,8 +784,8 @@ proxy_verify_desc *pvd_setup_initializers(char *cadir, unsigned int pvxd_flags)
         return NULL;
     }
 
-    proxy_verify_ctx_init(pvxd);
-    proxy_verify_init(pvd, pvxd);
+    canl_proxy_verify_ctx_init(pvxd);
+    canl_proxy_verify_init(pvd, pvxd);
 
     /* If cadir is not specified, do the best as to get the 
        standard CA certificates directory name */
@@ -784,20 +799,22 @@ proxy_verify_desc *pvd_setup_initializers(char *cadir, unsigned int pvxd_flags)
     else
         pvd->pvxd->certdir = strdup(cadir);
     pvd->pvxd->flags |= pvxd_flags;
+    if (ocsp_url)
+        pvd->pvxd->ocsp_url = strdup(ocsp_url);
     return pvd;
 }
 
-void pvd_destroy_initializers(void *data)
+void canl_pvd_destroy_initializers(void *data)
 {
-    proxy_verify_desc *pvd = (proxy_verify_desc *)data;
+    canl_proxy_verify_desc *pvd = (canl_proxy_verify_desc *)data;
 
     if (pvd) {
         if (pvd->pvxd)
-            proxy_verify_ctx_release(pvd->pvxd);
+            canl_proxy_verify_ctx_release(pvd->pvxd);
 
         free(pvd->pvxd);
         pvd->pvxd = NULL;
-        proxy_verify_release(pvd);
+        canl_proxy_verify_release(pvd);
 
         /* X509_STORE_CTX_free segfaults if passed a NULL store_ctx */
         if (pvd->cert_store)
